package info.edoardonosotti.android.tutorial.restfulws;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import android.app.Activity;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

/*
 * This demo app will connect to server REST API, download a list of available
 * packages, download them one by one, unpack zips and publish extracted text
 * content in a ListView.
 * Remember to set the correct base url in Consts.java!
 */
public class MainActivity extends Activity {
	
	private static String TAG = "MainActivity";
	
	private ArrayList<String> items = new ArrayList<String>();
	private ArrayAdapter<String> adapter;
	private JSONArray packages;
	private int currentPackage = 0;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        // Setup an adapter to programmatically add items to the ListView
        adapter = new ArrayAdapter<String>(this,
        		android.R.layout.simple_list_item_1,
        		items);
        ((ListView) findViewById(R.id.listView1)).setAdapter(adapter);
        
        // Check if the network is available before calling remote services
        if (checkNetwork()) {
        	getItems();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }
    
    // Launch a background task to call a remote method and get the JSON response
    public void getItems() {
    	String methodUrl = Consts.API_BASE + "getlist.php";
    	new RPCTask(new GetListHandler()).execute(methodUrl);
    }
    
    // Parse the JSON response
    public void parseList(String list) {
    	JSONObject json;
    	try {
			json = new JSONObject(new JSONTokener(list));
		} catch (JSONException e) {
			Log.e(TAG, e.getMessage());
			e.printStackTrace();
			return;
		}
    	
    	// Check for errors
    	if (json.optInt("status", -1) == 0) {
    		try {
				packages = json.getJSONArray("packages");
				// Begin the download procedure
				downloadPackage();
			} catch (JSONException e) {
				Log.e(TAG, e.getMessage());
				e.printStackTrace();
				return;
			}
    	} else {
    		// TODO - Handle error
    	}
    }
    
    public void downloadPackage() {
    	// Check if packages are available
    	if (packages != null && packages.length() > 0 && currentPackage < packages.length()) {
    		JSONObject item;
    		try {
				item = packages.getJSONObject(currentPackage);
			} catch (JSONException e) {
				Log.e(TAG, e.getMessage());
				e.printStackTrace();
				return;
			}
    		
    		// Check if the current package is compatibile with this app version
    		if (item.optInt("appversion", -1) == Consts.APP_VERSION) {
    			File path;
    			// Prefer external storage (sdcard) if available
    			if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
    				path = getExternalFilesDir(null);
    			} else {
    				path = getFilesDir();
    			}
    			File dest = new File(path, item.optString("file", ""));
				String source = Consts.REPO_BASE + item.optString("file", "");
				// Start a background download task
				new DownloadTask(new DownloadHandler()).execute(source, dest.getAbsolutePath());
    		} else {
    			// Skip to next
    			currentPackage++;
    			downloadPackage();
    		}
    	}
    	else if (packages != null && currentPackage >= packages.length()) {
    		Toast.makeText(MainActivity.this, getString(R.string.download_complete), Toast.LENGTH_SHORT).show();
    	}
    }
    
    public void addListItem(String item) {
    	items.add(item);
    	adapter.notifyDataSetChanged();
    }
    
    public void readTextFile(File file) {
    	StringBuilder text = new StringBuilder();

    	try {
    	    BufferedReader br = new BufferedReader(new FileReader(file));
    	    String line;

    	    while ((line = br.readLine()) != null) {
    	        text.append(line);
    	        text.append('\n');
    	    }
    	    
    	    br.close();
    	}
    	catch (IOException e) {
    	    Log.e(TAG, e.getMessage());
    	    e.printStackTrace();
    	    Toast.makeText(MainActivity.this, getString(R.string.read_file_error), Toast.LENGTH_SHORT).show();
    	    return;
    	}
    	
    	addListItem(text.toString());
    }
    
    // Always check network status before calling remote methods
    private boolean checkNetwork() {
        ConnectivityManager connectivityManager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo n = connectivityManager.getActiveNetworkInfo();
        
        // If return value is null, no network is available
        if (n == null) {
        	Toast.makeText(MainActivity.this, getString(R.string.network_unavailable), Toast.LENGTH_SHORT).show();
        	return false;
        }
        
        // Network may be available, but not connected
        if (n.isAvailable() && !n.isConnected()) {
        	Toast.makeText(MainActivity.this, getString(R.string.network_offline), Toast.LENGTH_SHORT).show();
        }
        
        // Roaming may be expensive, so it's ok to warn the user
        if (n.isRoaming()) {
        	Toast.makeText(MainActivity.this, getString(R.string.roaming_network), Toast.LENGTH_SHORT).show();
        }
        
        return n.isConnected();
    }
    
    // This handler will be called as soon as the remote method call will be completed.
    class GetListHandler extends Handler {

		@Override
		public void handleMessage(Message msg) {
			super.handleMessage(msg);
			Bundle b = msg.getData();
			String json = b.getString(RPCTask.RESULT);
			if (json != null) {
				parseList(json);
			} else {
				Toast.makeText(MainActivity.this, getString(R.string.ws_list_error), Toast.LENGTH_SHORT).show();
			}
		}
    	
    }
    
    // This handler will be called each time a download is completed.
    class DownloadHandler extends Handler {

		@Override
		public void handleMessage(Message msg) {
			super.handleMessage(msg);
			Bundle b = msg.getData();
			String path = b.getString(DownloadTask.RESULT);
			if (path != null) {
				File downloaded = new File(path, "content.txt");
				readTextFile(downloaded);
			} else {
				Toast.makeText(MainActivity.this, getString(R.string.ws_download_error), Toast.LENGTH_SHORT).show();
			}
			
			// Skip to next
			currentPackage++;
			downloadPackage();
		}
    	
    }
}